package com.dgtly.system.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.dgtly.common.core.domain.Ztree;
import com.dgtly.common.utils.DateUtils;
import com.dgtly.common.utils.StringUtils;
import com.dgtly.common.utils.UserIdentityUtil;
import com.dgtly.system.domain.*;
import com.dgtly.system.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.dgtly.system.service.ISysUserOrderAuthorService;
import com.dgtly.common.core.text.Convert;
import org.springframework.transaction.annotation.Transactional;

/**
 * 权限Service业务层处理
 * 
 * @author dgtly
 * @date 2022-03-22
 */
@Service
public class SysUserOrderAuthorServiceImpl implements ISysUserOrderAuthorService 
{
    @Autowired
    private SysUserOrderAuthorMapper sysUserOrderAuthorMapper;

    @Autowired
    private SysUserAuthorRelMapper userAuthorRelMapper;

    @Autowired
    private SysUserExtMapper userExtMapper;

    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private SysUserQuitMapper userQuitMapper;
    @Autowired
    private AssRelcustomerinfoMapper relcustomerinfoMapper;
    @Autowired
    private CustomerMapper customersMapper;

    /**
     * 查询权限
     * 
     * @param authorId 权限ID
     * @return 权限
     */
    @Override
    public SysUserOrderAuthor selectSysUserOrderAuthorById(Long authorId)
    {
        return sysUserOrderAuthorMapper.selectSysUserOrderAuthorById(authorId);
    }

    /**
     * 查询权限列表
     * 
     * @param sysUserOrderAuthor 权限
     * @return 权限
     */
    @Override
    public List<SysUserOrderAuthor> selectSysUserOrderAuthorList(SysUserOrderAuthor sysUserOrderAuthor)
    {
        return sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(sysUserOrderAuthor);
    }

    /**
     * 新增权限
     * 
     * @param sysUserOrderAuthor 权限
     * @return 结果
     */
    @Override
    public int insertSysUserOrderAuthor(SysUserOrderAuthor sysUserOrderAuthor)
    {
        sysUserOrderAuthor.setCreateTime(DateUtils.getNowDate());
        return sysUserOrderAuthorMapper.insertSysUserOrderAuthor(sysUserOrderAuthor);
    }

    /**
     * 修改权限
     * 
     * @param sysUserOrderAuthor 权限
     * @return 结果
     */
    @Override
    public int updateSysUserOrderAuthor(SysUserOrderAuthor sysUserOrderAuthor)
    {
        sysUserOrderAuthor.setUpdateTime(DateUtils.getNowDate());
        return sysUserOrderAuthorMapper.updateSysUserOrderAuthor(sysUserOrderAuthor);
    }

    /**
     * 删除权限对象
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteSysUserOrderAuthorByIds(String ids)
    {
        return sysUserOrderAuthorMapper.deleteSysUserOrderAuthorByIds(Convert.toStrArray(ids));
    }

    /**
     * 删除权限信息
     * 
     * @param authorId 权限ID
     * @return 结果
     */
    @Override
    public int deleteSysUserOrderAuthorById(Long authorId)
    {
        return sysUserOrderAuthorMapper.deleteSysUserOrderAuthorById(authorId);
    }

    @Override
    @Transactional
    public int insertRel(SysUserAuthorRel role)
    {
        /*1、查询所有一级节点*/
        String identity = "";
        SysUserExt sysUserExt = new SysUserExt();
        sysUserExt.setUserId(role.getUserId());
        for (Long authorId : role.getAuthorIds()){
            SysUserOrderAuthor sysUserOrderAuthor = sysUserOrderAuthorMapper.selectSysUserOrderAuthorById(authorId);
            if (sysUserOrderAuthor.getParentId()==1L){
                if(UserIdentityUtil.UI_DIY_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_DIY+",";
                }else if(UserIdentityUtil.UI_CW_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_CW+",";
                }else if(UserIdentityUtil.UI_YSL_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_YSL+",";
                }else if(UserIdentityUtil.UI_FC_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_FC+",";
                }else if(UserIdentityUtil.UI_XLS_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_XLS+",";
                }else if(UserIdentityUtil.UI_SAF_NAME.equals(sysUserOrderAuthor.getAuthorName())){
                    identity+=UserIdentityUtil.UI_SAF+",";
                }
            }
        }
        if (!"".equals(identity)) {
            sysUserExt.setIdentity(identity.substring(0, identity.length() - 1));
        }
        /*判断节点对应身份进行人员身份修改*/
        userAuthorRelMapper.deleteSysUserAuthorRelById(role.getUserId());
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysUserAuthorRel> list = new ArrayList<SysUserAuthorRel>();
        for (Long authorId : role.getAuthorIds())
        {
            SysUserAuthorRel rd = new SysUserAuthorRel();
            rd.setUserId(role.getUserId());
            rd.setAuthorId(authorId);
            list.add(rd);
        }
        if (list.size() > 0)
        {
            rows = userAuthorRelMapper.batchRel(list);
            if (!"".equals(identity)) {
                rows = userExtMapper.updateSysUserExt(sysUserExt);
            }
        }
        return rows;
    }

    /**
     * 查询权限树列表
     * 
     * @return 所有权限信息
     */
    @Override
    public List<Ztree> selectSysUserOrderAuthorTree()
    {
        List<SysUserOrderAuthor> sysUserOrderAuthorList = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Ztree> ztrees = new ArrayList<Ztree>();
        for (SysUserOrderAuthor sysUserOrderAuthor : sysUserOrderAuthorList)
        {
            Ztree ztree = new Ztree();
            ztree.setId(sysUserOrderAuthor.getAuthorId());
            ztree.setpId(sysUserOrderAuthor.getParentId());
            ztree.setName(sysUserOrderAuthor.getAuthorName());
            ztree.setTitle(sysUserOrderAuthor.getAuthorName());
            ztrees.add(ztree);
        }
        return ztrees;
    }

    public List<Ztree> userAuthorTreeData(Long userId) {
        List<Ztree> ztrees = new ArrayList<Ztree>();
        List<SysUserOrderAuthor> menuList = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Long> user = sysUserOrderAuthorMapper.selectAuthorTree(userId);
        ztrees = initTree(menuList, user);

        getAuthorByLevel();//todo
        return ztrees;
    }

    public List<Ztree> userAuthorTreeDataFmt(Long userId) {
        List<Ztree> ztrees = new ArrayList<Ztree>();
        List<SysUserOrderAuthor> menuList = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Long> user = sysUserOrderAuthorMapper.selectAuthorTree(userId);
        ztrees = initTree(menuList, user);
        ztrees = buildTree(ztrees, 0l);
        return ztrees;
    }

    //把一个List转成树
    static List<Ztree> buildTree(List<Ztree> list,Long pid){
        List<Ztree> tree=new ArrayList<>();
        for(Ztree node:list){
            if(Objects.equals(node.getpId(),pid)){
                tree.add(findChild(node,list));
            }
        }
        return tree;
    }

    static Ztree findChild(Ztree node, List<Ztree> list){
        for(Ztree n:list){
            if(Objects.equals(n.getpId(),node.getId())){
                if(node.getChildren() == null){
                    node.setChildren(new ArrayList<Ztree>());
                }
                node.getChildren().add(findChild(n,list));
            }
        }
        return node;
    }

    /**
     * 对象转树
     * @param sysUserOrderAuthorList
     * @return
     */
    public List<Ztree> initTree(List<SysUserOrderAuthor> sysUserOrderAuthorList, List<Long> roleMenuList)
    {
        List<Ztree> ztrees = new ArrayList<Ztree>();
        boolean isCheck = StringUtils.isNotNull(roleMenuList);
        for (SysUserOrderAuthor menu : sysUserOrderAuthorList)
        {
            Ztree ztree = new Ztree();
            ztree.setId(menu.getAuthorId());
            ztree.setpId(menu.getParentId());
            ztree.setName(menu.getAuthorName());
            ztree.setTitle(menu.getAuthorName());
            ztree.setCode(menu.getCode());
            if (isCheck)
            {
                ztree.setChecked(roleMenuList.contains(menu.getAuthorId()));
            }
            ztree.setOpen(true);
            ztrees.add(ztree);
        }
        return ztrees;
    }

    public void userAuthorTask(){

    }

    public void getAuthorByLevel(){
        List<SysUserOrderAuthor> list = sysUserOrderAuthorMapper.selectSysUserOrderAuthorList(new SysUserOrderAuthor());
        List<Long> ids = new ArrayList<>();
        getParentAuthor(ids, list, 8l);
        System.out.println(ids);
    }

    public void getParentAuthor(List<Long> ids, List<SysUserOrderAuthor> list, Long id){
        for (SysUserOrderAuthor sysUserOrderAuthor : list) {
            Long itemId = sysUserOrderAuthor.getAuthorId();
            if(itemId.equals(0l)){
                ids.add(itemId);
            }else if(itemId.equals(id)){
                ids.add(itemId);
                getParentAuthor(ids, list , sysUserOrderAuthor.getParentId());
            }
        }
    }

    @Override
    public void initUserAuthor() {
        List<SysUserOrderAuthor> userOrderAuthorList = new ArrayList<>();
        //diy
        List<SysUserOrderAuthor> userOrderAuthorDiyList = sysUserOrderAuthorMapper.selectAuthorIdByCode("tuc_diy");
        List<Long> diyauthList=userOrderAuthorDiyList.stream().map(SysUserOrderAuthor::getAuthorId).collect(Collectors.toList());
        List<SysUserOrderAuthor> diyrole=sysUserOrderAuthorMapper.selectByParentId(diyauthList);
        userOrderAuthorDiyList.addAll(diyrole);
        //cw
        List<SysUserOrderAuthor> userOrderAuthorCwList = sysUserOrderAuthorMapper.selectAuthorIdByCode("tuc_cw");
        List<Long> cwauthList=userOrderAuthorCwList.stream().map(SysUserOrderAuthor::getAuthorId).collect(Collectors.toList());
        List<SysUserOrderAuthor> cwrole=sysUserOrderAuthorMapper.selectByParentId(cwauthList);
        userOrderAuthorCwList.addAll(cwrole);
       //ysl
        List<SysUserOrderAuthor> userOrderAuthorYSLList = sysUserOrderAuthorMapper.selectAuthorIdByCode("YSL");
        List<Long> yslauthList=userOrderAuthorYSLList.stream().map(SysUserOrderAuthor::getAuthorId).collect(Collectors.toList());
        List<SysUserOrderAuthor> yslrole=sysUserOrderAuthorMapper.selectByParentId(yslauthList);
        userOrderAuthorCwList.addAll(yslrole);

        userExtMapper.selectUserExtList().forEach(sysUserExt -> {
            String sales = sysUserExt.getSalesLevel();
            if (sales != null) {
            int index = sales.indexOf("_");
            String intNumber = sales.substring(0, index);
            if (intNumber.equals("cw")) {
                userOrderAuthorList.clear();
                userOrderAuthorList.addAll(userOrderAuthorCwList);
            } else if (intNumber.equals("diy")||intNumber.equals("company") || intNumber.equals("office") || intNumber.equals("emp")
                    || intNumber.equals("saledept") || intNumber.equals("suboffice")|| intNumber.equals("customer")) {
                userOrderAuthorList.clear();
                userOrderAuthorList.addAll(userOrderAuthorDiyList);
            } else if (intNumber.equals("ysl")) {
                userOrderAuthorList.clear();
                userOrderAuthorList.addAll(userOrderAuthorYSLList);
            }
            if(userOrderAuthorList.size()>0){
            List<Long> userole = userAuthorRelMapper.selectSysUserAuthorRelListById(sysUserExt.getUserId());
            if (userole ==null ||userole.size() == 0) {
                SysUserAuthorRel role = new SysUserAuthorRel();
                role.setUserId(sysUserExt.getUserId());
                role.setAuthorId(userOrderAuthorList.get(0).getParentId());
                userAuthorRelMapper.insertSysUserAuthorRel(role);
                userOrderAuthorList.forEach(sysUserOrderAuthor -> {
                            SysUserAuthorRel userRel = new SysUserAuthorRel();
                            userRel.setUserId(sysUserExt.getUserId());
                            userRel.setAuthorId(sysUserOrderAuthor.getAuthorId());
                            userAuthorRelMapper.insertSysUserAuthorRel(userRel);
                        });
            }else{
                userole.remove(0);
                List<Long> uinon=userOrderAuthorList.stream().map(SysUserOrderAuthor::getAuthorId).collect(Collectors.toList());
                uinon.removeAll(userole);
                if(uinon.size()>0){
                List<Long> newrole=uinon.stream().distinct().collect(Collectors.toList());
                    newrole.forEach(role -> {
                    SysUserAuthorRel userRel = new SysUserAuthorRel();
                    userRel.setUserId(sysUserExt.getUserId());
                    userRel.setAuthorId(role);
                    userAuthorRelMapper.insertSysUserAuthorRel(userRel);
                });
            }
            }
            }
            }
        });

    }

    @Override
    public void quitUser() {
        List<SysUser> userList=userMapper.selectUser();
        Set<String> SapEmployeeIdList=userQuitMapper.selectAllUserSapEmployeeId();
         Set<String> employList=SapEmployeeIdList.stream().map(String::toUpperCase).collect(Collectors.toSet());
         for (SysUser sysUser :userList
                 ) {
             SysUserExt userExt = userExtMapper.selectSysUserExtById(sysUser.getUserId());
             if (userExt !=null && userExt.getSapEmployeeId() != null && userExt.getSapEmployeeId() !="") {
                 if(employList!=null && employList.size()>0){
                 if (employList.contains(userExt.getSapEmployeeId().toUpperCase())) {
                     sysUser.setQuit("2");
                     userMapper.updateUser(sysUser);
                 }
                 }
             }
         }

    }

    @Override
    public void updateCustomers() {
            List<AssRelcustomerinfo> list=relcustomerinfoMapper.selectCustomerNameAndCode();
            List<Customer> customersList=new ArrayList<>();
            if(list != null && list.size()>0){
                for (AssRelcustomerinfo info:list
                     ) {
                   Customer customers= customersMapper.selectCustomersByChainsCode(info.getMainkunnr());
                   if(customers==null){
                       Customer customer=new Customer();
                       customer.setChainsName(info.getMainname1());
                       customer.setChainsCode(info.getMainkunnr());
                       customersList.add(customer);
                   }
                }
                customersList.forEach(customers ->{
                    customersMapper.insertCustomers(customers);
                } );
            }
    }

}

